const jwt = require('jsonwebtoken');
const bcrypt = require('bcrypt');
const db = require('./db/db');
const { secret, expiresIn } = require('./config/jwt');

const login = async (body) =>{
    let email = body["lg-email"]
    let password = body["lg-password"]
    let loginResult = false;
    let token = null;
    let genericFail = "Correo electrónico o contraseña incorrectos";
    try {
        if(email && password)
        {        
            const queryForUsers = 'SELECT users.*, userRoles.name as roleName  FROM users left join userRoles on users.userRole_id = userRoles.id WHERE email = ? and is_verified = 1 and is_active = 1;';             
            const user = await db.query(queryForUsers, [email]);            
            if(user && user.length)
            {
                const match = await bcrypt.compare(password, user[0].password);
                if(match)
                {                    
                    loginResult = true;
                    token = jwt.sign({ userId: user[0].id, type: user[0].roleName, roleId: user[0].userRole_id , userName: user[0].name }, secret, { expiresIn });
                }
                return { result: loginResult, msj: loginResult ? null: genericFail , email, password, type: user[0].roleName, roleId: user[0].userRole_id , userId: user[0].id, token }
            }            
            else
            {    
                return { result: false, msj: "Correo electrónico o contraseña incorrectos" , email, password, type:null, userId: 0, token }
            }          
        }
        else 
        {
            let msj = null;
            if(email)
            {
                msj = "Contraseña no ingresada"
            }
            else if(password)
            {
                msj = "Correo electrónico no ingresado"
            }
            else
            {
                msj = "Correo electrónico y Contraseña no ingresados"
            }
            return { result: false, msj , email, password, type:null, userId: 0, token  }
        }
    } catch (error) {
        console.log(error)
        return { result: false, msj: "Correo electrónico y Contraseña no ingresados" , email, password, type:null, userId: 0, token  }
    }
    
}
const isAuthenticated  = async (req, res,  next) => {

    if (req.session.token) 
    {
        const decoded = jwt.verify(req.session.token, secret);
        if(decoded)
        {
         
            res.locals.userId = decoded.userId;
            res.locals.userType = decoded.type;
            res.locals.roleId = decoded.roleId;
            res.locals.userName = decoded.userName;
            next();
        }
        else
        {
            res.locals.userId = null;
            res.locals.userType = null;
            res.locals.userName = null;
            res.locals.roleId = null;
            return res.redirect('/login');
        }
    }
    else
    {
        res.locals.userId = null;
        res.locals.userType = null;
        res.locals.userName = null;
        res.locals.roleId = null;
        return res.redirect('/login');
    } 
  }

  const isAuthenticatedNoRedirect  = async (req, res,  next) => {

    if (req.session.token) 
    {
        const decoded = jwt.verify(req.session.token, secret);
        if(decoded)
        {
         
            res.locals.userId = decoded.userId;
            res.locals.userType = decoded.type;
            res.locals.roleId = decoded.roleId;
            res.locals.userName = decoded.userName;
            next();
        }
        else
        {
            res.locals.userId = null;
            res.locals.userType = null;
            res.locals.userName = null;
            res.locals.roleId = null;     
            next();       
        }
    }
    else
    {
        res.locals.userId = null;
        res.locals.userType = null;
        res.locals.userName = null;
        res.locals.roleId = null;
        next();        
    } 
  }



  const getDecodedDataFromToken = async(req) =>
  {
    const decoded = jwt.verify(req.session.token, secret);
    return decoded;
  }
  const isNotAuthenticated = async (req, res, next) => {
    if (req.session.token)
    {
       res.redirect('/menu');
    }
    else
    {
      next();
    }
  }
const logout = async(token) =>
{
    await jwt.destroy(token);
}
module.exports ={
    login,
    isAuthenticated,
     isNotAuthenticated,
     logout,
     getDecodedDataFromToken,
     isAuthenticatedNoRedirect
}