const moment = require('moment');
const  userModelo  = require('../models/User');
const  adminUserModelo  = require('../models/adminUser');
const  horariosModelo  = require('../models/horarios');
const  datesModelo  = require('../models/dates');
const  sesiongrabacionModelo  = require('../models/sesiongrabacion');
const  asisitentesmusicosModel  = require('../models/asisitentesmusicos');
const  asisitentesNomusicosModel  = require('../models/asistentesNoMusicos');
const  equipoestudioModelo  = require('../models/equipoestudio');
const  serviciosAdicionalesModelo  = require('../models/serviciosAdicionales');
const  serviciosModelo  = require('../models/servicios');
const  cotizacionesModelo  = require('../models/cotizaciones');
const  pdfGenerator  = require('../utils/pdfGenerator');
const  sesiondegrabacion  = require('../services/sesiondegrabacion');
const locationsModelo = require('../models/locations')
const  percentagesModelo  = require('../models/percentages');
const  preceivedPaymentsModelo  = require('../models/receivedPayments');


const loadStep2 = async (req, res) =>{  
    let user = null;
    let sesionGrabacionToReturn = "";
    let userById = await userModelo.getUserById(req.params.id)
    let sesionGrabacion = await sesiongrabacionModelo.getActiveHorarios(req.params.sesionId)
    if(userById)
    {
        user = userById[0];
    }
    if(sesionGrabacion)
    {
        sesionGrabacionToReturn = sesionGrabacion[0]
    }    
    return res.render('./partials/_step2.html', { layout: false, user, sesionGrabacionToReturn });
}
const calendar = async (req, res) =>{
    let { month, year } =  req.params;     
    let result = await getDaysByMonthAndYear(req.params.month, req.params.year)
    moment().locale('es').format("YYYY-MM-DD")
    const valoresUnicos = result.diasDelMes.reduce((valores, objeto) => {
        if (!valores.includes(objeto.wk)) {
          valores.push(objeto.wk);
        }
        return valores;
      }, []);
    if(req.params.month != 12)
    {
        valoresUnicos.sort((a, b) => a - b);
    }
    return res.render('./partials/_calendar.html', 
    { 
        layout: false, 
        days: result.diasDelMes, 
        wks: valoresUnicos, 
        month: req.params.month, 
        year: req.params.year, 
        monthName: result.monthName, 
        currentYear: result.currentYear, 
        enableBack: result.enableBack        
    });
}
const getDaysByMonthAndYear = async (month, year) =>
{
    let enableDays = false;
    let enableBack = false;
    let currentDay = moment().date();
    let currentMonth = 1 + moment().month();
    let currentYear = moment().year();
  
    const primerDiaDelMes = moment([year,  month - 1]);;
    let monthName = moment(primerDiaDelMes).locale('es').format('MMMM');
    const ultimoDiaDelMes = primerDiaDelMes.clone().endOf('month');
    const diasDelMes = [];
    let diaActual = primerDiaDelMes.clone();
    let counter = 1;
    while (diaActual.isSameOrBefore(ultimoDiaDelMes)) {
        if(diaActual.isAfter(moment()))
        {
            enableDays = true;
        }
        if(year > currentYear)
        {
            enableBack = true;
        }
        else if((month > currentMonth) && (currentYear == year))
        {
            enableBack = true;
        }
        let isCurrentDay = (currentDay == counter && currentMonth == parseInt(month) && currentYear == parseInt(year)) ? true: false;        
        const nombreDiaSemana = diaActual.format('dddd');    
        diasDelMes.push({ dayName: nombreDiaSemana, dayNumerOfWeek: diaActual.day(), date: diaActual.format('DD-MM-YYYY'), dayNumer: diaActual.date(), wk: diaActual.format('w'), counter, isCurrentDay, enableDays})
        diaActual.add(1, 'days');
        counter++;
    }
    return { diasDelMes, monthName, currentYear: year, enableBack};
}

const getHorariosByDate = async (req, res) => {
    let activeServices = await serviciosModelo.getActiveServices();
    let horarios = await horariosModelo.getActiveHorarios()  
    let checkedHorarios = [];    
    let formatedDate = moment(req.params.date,  'YYYY-MM-DD').locale('es-us').format("ll");    
    if(req.params.id && req.params.date)
    {
        let dates = await horariosModelo.getDateId(req.params.date, req.params.id)
        if(dates && dates.length > 0)
        {
            checkedHorarios = await horariosModelo.getHorariosByDateId(dates[0].id)            
        }
    }
    return res.render('./partials/_horariosDisponibles.html', { layout: false, horarios, checkedHorarios, formatedDate, activeServices })    
}

const getHorariosByDateV2 = async (req, res) => {    
    let { date, id, locationId, serviceId } = req.params;
    let horarios = await horariosModelo.getActiveHorarios()  
    let checkedHorarios = [];  
    let services = [];  
    let formatedDate = moment(date,  'YYYY-MM-DD').locale('es-us').format("ll");    
    if(req.params.id && date)
    {
        let dates = await horariosModelo.getDateId(date, id)
        if(dates && dates.length > 0)
        {
            checkedHorarios = await horariosModelo.getHorariosByDateId(dates[0].id)            
        }
    }
    if(parseInt(serviceId) === 0)
    {
        services = await serviciosModelo.getActiveServices()
    }
    else
    {
       services = await serviciosModelo.getServiceById(serviceId)
    }

    return res.render('./partials/_horariosDisponiblesv2.html', { layout: false, horarios, checkedHorarios, formatedDate, services, locationId, serviceId, idsesiongrabacion: id  })    
}




const getHorariosToUpdate = async (req, res) => {    

    let { fechaId } = req.params;
    let dates = await datesModelo.getDateByFechaId(fechaId);
    let dateRow = dates[0];
    let checkedHorarios = [];  
    let horarios = await horariosModelo.getActiveHorarios();
    let bussyDates = await horariosModelo.getDateId(dateRow.date)
    let horariosAEditar = await horariosModelo.getHorariosByDateId(fechaId)
    if(bussyDates && bussyDates.length > 0)
    {
        checkedHorarios = await horariosModelo.getHorariosByDateId(bussyDates[0].id)            
    }    
    let formatedDate = moment(dateRow.date,  'YYYY-MM-DD').locale('es-us').format("ll");        
    let services = await serviciosModelo.getActiveServices()    
    return res.render('./partials/_horariosToUpdate.html', { layout: false, horarios, checkedHorarios, services, horariosAEditar, formatedDate, idsesiongrabacion: dateRow.sesiongrabacion_id })    
}




const getDraftHorariosa = async (req, res) => {
    let newList = await sesiondegrabacion.getFechaYHorarioResumidos(req.params.id)    
    return res.render('./partials/_horariosSeleccionados.html', { layout: false, newList})    
}
const getAsistentesMusicos = async (req, res) => {
    let musicos = await asisitentesmusicosModel.getMusicosById(req.params.id)
  
    return res.render('./partials/_musicosList.html', { layout: false, musicos})    
}
const getMusicToEdit = async (req, res) => {

    let musico = await asisitentesmusicosModel.getMusicoById(req.params.id)

    return res.render('./partials/_editMusic.html', { layout: false, musico: musico[0]})    
}
const getNoMusicToEdit = async (req, res) => {

    let musico = await asisitentesNomusicosModel.getMusicoById(req.params.id)

    return res.render('./partials/_editNoMusic.html', { layout: false, musico: musico[0]})    
}
const getAsistentesNoMusicos = async (req, res) => {
    let musicos = await asisitentesNomusicosModel.getMusicosById(req.params.id)

    return res.render('./partials/_noMusicList.html', { layout: false, musicos})    
}
const getEquipoestudio = async (req, res) => {
    let equipoEstudio = await equipoestudioModelo.getAllEquipoEstudioById(req.params.id)

    return res.render('./partials/_equipoAdicional.html', { layout: false, equipoEstudio})    
}
const getEquipoestudioEdit = async (req, res) => {
    let equipoEstudio = await equipoestudioModelo.getEquipoEstudioById(req.params.id)
    return res.render('./partials/_editEquipoAdicional.html', { layout: false, equipoEstudio: equipoEstudio[0]})    
}

const getStep1 = async (req, res) => {
    let sesion_id = req.params.id;
    let sesion =  await sesiongrabacionModelo.getSesiongrabacion(sesion_id)
    let user = null;
    if(sesion)
    {
        user = await userModelo.getUserById(sesion[0].user_id)
    }
    return res.render('./partials/_stp1.html', { layout: false, sesion: sesion[0], sesion_id, user: user[0] })    
}
const getStep2 = async (req, res) => {
    let sesion_id = req.params.id;
    let newList = await sesiondegrabacion.getFechaYHorarioResumidos(sesion_id) 

    return res.render('./partials/_stp2.html', { layout: false, newList})    
}
const getStep3 = async (req, res) => {
    let sesion_id = req.params.id;
    let musicos = await asisitentesmusicosModel.getMusicosById(sesion_id)  
    let nomusicos = await asisitentesNomusicosModel.getMusicosById(sesion_id)

    return res.render('./partials/_stp3.html', { layout: false, musicos, nomusicos})    
}

const getStep4 = async (req, res) => {
    let sesion_id = req.params.id;

    let checkedServices = await serviciosModelo.getCheckedServicesJoined(sesion_id)
    let checkedServicesAd = await serviciosAdicionalesModelo.getCheckedServicesJoined(sesion_id)    
    return res.render('./partials/_stp4.html', { layout: false, checkedServices, checkedServicesAd })    
}

const getStep5 = async (req, res) => {
    let sesion_id = req.params.id;
    let equipoEstudio = await equipoestudioModelo.getAllEquipoEstudioById(req.params.id)
    return res.render('./partials/_stp5.html', { layout: false, equipoEstudio })    
}






const downloadCotizacion = async (req, res) => {
    let cotizacionId = req.params.cotizacionId
    let list = await cotizacionesModelo.getCotizacionElementosForPdf(cotizacionId);
    let customer = await cotizacionesModelo.getCustomerDatForPdf(cotizacionId);
    let { totalsTable, currencyMsj } = await cotizacionesModelo.getTotalsForPdf(cotizacionId);
    let msj = await cotizacionesModelo.getCotizacionValidity();
    let data = await cotizacionesModelo.getContactTable();    
    let { pdfsPath, fileName } = await pdfGenerator.generatePdf(list, data, customer, totalsTable, msj, currencyMsj);    
    res.download(`.${pdfsPath}${fileName}.pdf`, `${fileName}.pdf`, (err) => {
        if (err) {         
          console.error('Error al descargar el archivo:', err);
          res.status(500).send('Error interno del servidor');
        }  else {
            pdfGenerator.eliminarArchivo(`${pdfsPath}${fileName}.pdf`);
        }
  
    });
}

const passwordResetForm = async (req, res) => {
    let { userId, ut } = req.params;
    let enableForm = true;
    let email = null;    
    let userName = null;
    if(parseInt(ut) == 1)
    {
        let user = await userModelo.getUserByIdForPasswordReset(userId);
        enableForm = user.length > 0 ? true : false;
        email = enableForm ?  user[0].email: "";
        userName = enableForm ?  user[0].name: "";
    } 
    else
    {
        let user = await adminUserModelo.getUserByIdForPasswordReset(userId);
        enableForm = user.length > 0 ? true : false;
        email = enableForm ?  user[0].email: "";
        userName = enableForm ?  user[0].name: "";
    }
    return res.render('./auth/passwordResetForm.html', { enableForm, enablePostMsj: false, userId, email, userName, ut })     
}

const getFechaYHorarioResumidos = async (req, res) => {
    
    return res.json(await sesiondegrabacion.getFechaYHorarioResumidos())
}

const acceptSesion = async (req, res) => {
    let id = req.params.id;
    let approvedStatusId = 4;
    await sesiongrabacionModelo.updateCitaToStatus(id,approvedStatusId);
    let newList = await sesiondegrabacion.getFechaYHorarioResumidos(id)
    return res.render('./acceptSesion.html', { newList })  
}


const test = async (req, res) => {
    await sesiondegrabacion.generateAutomaticQuote(119)
    return res.json({ result: true })
}


const changeSesionStatusApprove = async (req, res) => {
    let { sesiongrabacion_id, user_id } = req.params;

    let approvedStatusId = 2;
    await sesiongrabacionModelo.updateCitaToStatus(sesiongrabacion_id, approvedStatusId);
    if(user_id)
    {
        await sesiongrabacionModelo.updateInChargePerson(sesiongrabacion_id, user_id);
    }
    return res.redirect(`/menuv2`)
  }
  
  
  const changeSesionStatusEdit = async (req, res) => {
    let sesiongrabacion_id = req.params.sesiongrabacion_id;
    let rejectedStatusId = 1;
    await sesiongrabacionModelo.updateCitaToStatus(sesiongrabacion_id, rejectedStatusId)
    return res.redirect(`/menuv2`)
  }
  const getCalendars = async (req, res) => {
    try {
        let locations = await locationsModelo.getActiveLocations();
        let { year, month } = req.params;

        let result = await getDaysByMonthAndYear(month, year)
        moment().locale('es').format("YYYY-MM-DD")
        const valoresUnicos = result.diasDelMes.reduce((valores, objeto) => {
            if (!valores.includes(objeto.wk)) {
            valores.push(objeto.wk);
            }
            return valores;
        }, []);
        if(month != 12)
        {
            valoresUnicos.sort((a, b) => a - b);
        }
        let busyDates = await datesModelo.getBusyDates(year, month);
        return res.render('./partials/_calendars.html', 
        { 
            layout: false, 
            locations,
            days: result.diasDelMes, 
            monthName: result.monthName, 
            currentYear: result.currentYear, 
            wks: valoresUnicos, 
            busyDates,
            year,
            month
        })    
    } catch (error) {
        console.log("getCalendars", error)
    }
    
  }  
  const getBusyDate = async (req, res) => {
    let { location_id, year, month, dayNumer } = req.params; 
    let busyDate = await datesModelo.getBusyDatesByIds(parseInt(location_id), parseInt(year), parseInt(month), parseInt(dayNumer))    
    return res.render('./partials/_horariosOcupadosPorFecha.html',{    layout: false, busyDate });
  }

  const deleteCotizacion = async (req, res) => {
    let { id, idsesiongrabacion } = req.params;
    return res.redirect(`/cotizacionesMenu/${idsesiongrabacion}`)
  }

  const getAllRecordsByQty = async (req, res) => {
    let { qty } = req.params;
    let list = await sesiongrabacionModelo.getAllSesionsgrabacionByQty(parseInt(qty));
    return res.json(list);
  }

  const getCalculator = async (req, res) => {
    let { year, month, user_id } = req.params;    
    let servicesToPay = await sesiongrabacionModelo.getServicesToPay(year, month, user_id)
    let data = await sesiondegrabacion.getDataToPayByUserAndService( year, month, user_id)
    let granTotal = 0;
    for(let dt of data)
    {
        granTotal += dt.total;
    }    
    return res.render('./partials/_calculator.html', { layout: false, servicesToPay, data, granTotal });
  }

const getCalculatorV2 = async (req, res) => {
    let { year, month, user_id } = req.params;    
    let data = await sesiondegrabacion.getSesionsToPay(year, month, user_id)    
    const sumResult = await processCotizationsData(data);    
    const receivedPayments = await preceivedPaymentsModelo.getReceivedPayments(year, month, user_id);    
    return res.render('./partials/_calculatorv2.html', { layout: false, data, sumResult, receivedPayments });
}





const normalizeString = (str) => 
    str.normalize('NFD').replace(/[\u0300-\u036f]/g, '').toLowerCase();

// Función principal para procesar data y devolver un solo array con todos los resultados
const processCotizationsData = (data) => {
    const sumsByJustification = {};
    const originalJustifications = new Set();
    let totalCotizationsSum = 0;
    let totalPercentagesSum = 0;

    data.forEach(record => {
        record.cotizaciones.forEach(cotizacion => {
            // Sumar el total de todas las cotizaciones
            totalCotizationsSum += parseFloat(cotizacion.cotizacion.total);

            cotizacion.percentages.forEach(percentage => {
                // Normalizar la justification
                const normalizedJustification = normalizeString(percentage.justification);
                const toPay = parseFloat(percentage.toPay);

                // Sumar el toPay por justification normalizada
                if (sumsByJustification[normalizedJustification]) {
                    sumsByJustification[normalizedJustification] += toPay;
                } else {
                    sumsByJustification[normalizedJustification] = toPay;
                }

                // Sumar el total de toPay
                totalPercentagesSum += toPay;

                // Guardar justificaciones originales
                originalJustifications.add(percentage.justification);
            });
        });
    });

    // Crear el array de resultados con las sumas por justification
    const resultArray = Object.keys(sumsByJustification).map(justification => ({
        justification,
        sum_toPay: sumsByJustification[justification]
    }));

    // Comparar originales con normalizados para encontrar diferencias
    const mismatchedJustifications = Array.from(originalJustifications).filter(justification => {
        return !Object.keys(sumsByJustification).includes(normalizeString(justification));
    });

    // Calcular el total menos lo sumado de todos los porcentajes
    const totalMinusPercentages = totalCotizationsSum - totalPercentagesSum;

    // Resultado final en un solo array
    const finalResult = {
        summedPercentages: resultArray,
        mismatchedJustifications,
        totalCotizationsSum,
        totalMinusPercentages
    };

    return finalResult;
};

const getCalculatorV2Raw = async (req, res) => {
    let { year, month, user_id } = req.params;    
    let data = await sesiondegrabacion.getSesionsToPay(year, month, user_id)
    return res.json(data);
}

  const getAditionalServices = async(req, res) => {
    let { sesionId } = req.params;
    let listToReturn = await serviciosModelo.getAditionalServicesBy_SesiondegrabacionId(sesionId);
    return res.render('./partials/_aditionalServices.html', { layout: false, listToReturn});
  }


  const getAditionalServicesForDisplay = async(req, res) => {
    let { sesionId } = req.params;
    let listToReturn = await serviciosModelo.getAditionalServicesBy_SesiondegrabacionId(sesionId);
    return res.render('./partials/_aditionalServicesForDisplay.html', { layout: false, listToReturn});
  }



  const getAditionalServiceToEdit = async(req, res) => {
    let services = await serviciosModelo.getServiceById(req.params.id)
    return res.render('./partials/_editAditionalService.html', { layout: false, service: services[0] });
  }

  

const getPercentagesByCotizacionIdGeneral = async (req, res) => {
    let { id } = req.params;    
    let data = await percentagesModelo.getPercentagesByCotizacionIdGeneral(id);
    return res.render('./partials/_percentagesList.html', { layout: false, data });
}

const editPercentageIndiv = async (req, res) => {
    let { id } = req.params;    
    let percentage = await percentagesModelo.getPercentageById(id);
    return res.render('./partials/_editPercentageIndiv.html', { layout: false, data: percentage[0] });
}




const getReceivedPaymentById = async (req, res) => {
    let { id } = req.params;    
    let data = await preceivedPaymentsModelo.getReceivedPaymentById(id);
    let obj = data[0];
    return res.json(obj);
}


const getReceivedPayments = async (req, res) => {
    let { year, month, user_id } = req.params;    
    let data = await preceivedPaymentsModelo.getReceivedPayments(year, month, user_id);
    return res.json(data);
}


const editReceivedPayment = async (req, res) => {
    let { id } = req.params;    
    let receivedPayment = await preceivedPaymentsModelo.getReceivedPaymentById(id);
    return res.render('./partials/_editReceivedPayment.html', { layout: false, data: receivedPayment[0] });
}




module.exports = {
    loadStep2,
    calendar,
    getHorariosByDate,
    getHorariosByDateV2,
    getDraftHorariosa,
    getAsistentesMusicos,
    getMusicToEdit,
    getNoMusicToEdit,
    getAsistentesNoMusicos,
    getEquipoestudio,
    getEquipoestudioEdit,
    getStep1,
    getStep2,
    getStep3,
    getStep4,
    getStep5,
    downloadCotizacion,
    passwordResetForm,
    getFechaYHorarioResumidos,
    acceptSesion,
    getHorariosToUpdate,
    test,
    changeSesionStatusApprove,
    changeSesionStatusEdit,
    getCalendars,
    getBusyDate,
    deleteCotizacion,
    getAllRecordsByQty,
    getCalculator,
    getCalculatorV2,
    getAditionalServices,
    getAditionalServiceToEdit,
    getAditionalServicesForDisplay,
    getPercentagesByCotizacionIdGeneral,
    editPercentageIndiv,
    getCalculatorV2Raw,
    getReceivedPaymentById,
    getReceivedPayments,
    editReceivedPayment
    // getLocations
}