const mysqlConnection = require('../config/database');
const moment = require('moment');
const { CONTACTPHONE, CONTACTEMAIL, CONTACTDIRECCION } = require('../config');
const sesiongrabacionModelo = require('../models/sesiongrabacion');




const getCotizacionValidity = async() => {  
  moment.locale('es');
  const now = moment();
  const currentDay = now.date(); 
  const currentMonth = now.format('MMMM'); 
  const currentYear = now.year(); 
  const nowPlusOneMonth = now.add(1, 'months');
  const endDay = nowPlusOneMonth.date(); 
  const endMonth = nowPlusOneMonth.format('MMMM'); 
  const endYear = nowPlusOneMonth.year(); 
  let msj = `Esta cotización es válida desde el ${currentDay} de ${currentMonth} de ${currentYear} hasta el ${endDay} de ${endMonth} de ${endYear}. Para cualquier pregunta o aclaración, no dudes en ponerte en contacto a través del correo electrónico ${CONTACTEMAIL} o llamando por teléfono al ${CONTACTPHONE}.`
  return msj;
}

const getContactTable = async() =>{  
  const formato = 'MMMM D, YYYY h:mm A';
  moment.locale('es');
  let data = [[moment().format(formato).replace(/\b\w/g, (l) => l.toUpperCase()), CONTACTPHONE, CONTACTEMAIL,CONTACTDIRECCION]]    
  return data;
}

const getCotizacionElementosForPdf = async(cotizacionId) => {
  let elementos = await getCotizacionElementos(cotizacionId);
  let listToReturn = [];
  for(let element of elementos)
  {
     let obj = [];
     obj.push(element.productoOServicio);
     obj.push(element.cantidad.toLocaleString('es-MX'));
     obj.push(`$ ${element.precioUnitario.toLocaleString('es-MX')} `);
     obj.push(`$ ${element.total.toLocaleString('es-MX')}`);
     listToReturn.push(obj)
  }
  return listToReturn;
}

const getTotalsForPdf = async(cotizacionId) => {
  let toReturn = [];
  const { subTotal, iva, total, vatCalculated, tax, currency, currencyMsj} = await getTotals(cotizacionId);
  if(vatCalculated)
  {
    toReturn.push(["Subtotal", `$ ${subTotal.toLocaleString('es-MX')}`]) 
    toReturn.push([`Iva (${tax.toLocaleString('es-MX') } %)`, `$ ${iva}`])
    toReturn.push([`Total`, `$ ${total.toLocaleString('es-MX') } ${currency}`])         
  }
  else 
  {
    toReturn.push(["Subtotal", `$ ${subTotal.toLocaleString('es-MX')}`]) 
    toReturn.push([`Iva`, `NA`])
    toReturn.push([`Total`, `$ ${subTotal.toLocaleString('es-MX')} ${currency}`])     
  }
  return { totalsTable: toReturn, currencyMsj };
}

const getTotals = async (cotizacionId) => {
  let cotizacion = await getCotizacion(cotizacionId);
  let elementos = await getCotizacionElementos(cotizacionId);
  let subTotal = 0;
  let iva = 0;
  let tax = parseFloat(cotizacion[0].tax);
  let total = 0;
  let vatCalculated = false;
  let currencyMsj = cotizacion[0].currency == 1 ? 
  `Esta cotización está expresada exclusivamente en pesos mexicanos (MXN). Los montos indicados reflejan la moneda local y están sujetos a cambios sin previo aviso.` : 
  `Esta cotización está expresada exclusivamente en dólares estadounidenses (USD). Los montos indicados están denominados en la moneda mencionada y pueden estar sujetos a cambios sin previo aviso.`

  for(let element of elementos)
  {
    subTotal = subTotal + multiplicarConDecimal(element.precioUnitario, element.cantidad);
  }
  if(tax > 0)
  {
    iva = parseFloat(subTotal) / parseFloat(100) * tax;
    total = subTotal + iva;
    vatCalculated = true;    
  }
  return { subTotal, iva, total, vatCalculated, tax, currency: cotizacion[0].currency == 1 ? "MXN" : "USD", currencyMsj }
}



const getCustomerDatForPdf = async(cotizacionId) => {
  let cotizacion = await getCotizacion(cotizacionId);
  let data = [];
  let destinatarioNombre = cotizacion[0].destinatarioNombre ?? "";
  let destinatarioCorreo = cotizacion[0].destinatarioCorreo ?? "";
  let destinatarioTelefono = cotizacion[0].destinatarioTelefono ?? "";
  data.push(destinatarioNombre);
  data.push(destinatarioCorreo);
  data.push(destinatarioTelefono);
  return [ data ]
}

const getAllCotizacionData = async(cotizacionId) => {
  let objToReturn = { cotizacion: null, elementos: [] }
  let cotizacion = await getCotizacion(cotizacionId);
  objToReturn.cotizacion = cotizacion[0];  
  objToReturn.elementos = await getCotizacionElementos(cotizacionId);
  let total = 0;
  for(let element of objToReturn.elementos)
  {
    total = total + element.total;
  }
  objToReturn.cotizacion.total = total;
  objToReturn["percentages"] = [];
  objToReturn.percentages = await getPercentages(cotizacionId, objToReturn.cotizacion.total);
  return objToReturn;
}

const getPercentages = async(cotizacionId, total) => {
    if(!total || total == 0)
    {
      return [];
    }
    
    let toReturn = [];
    let percentages = await getPercentagesByCotizacionId(cotizacionId);
    for(let item of percentages)
    {
      console.log({total, percentage: item.percentage })
       let toPay = item.percentage/100 * total;
       toReturn.push({ toPay: toPay ? toPay.toFixed(2): 0, justification: item.justification, percentage: item.percentage ? item.percentage.toFixed(1): 0 });
    }
    return toReturn;
}

getPercentagesByCotizacionId = (cotizacionId) => {
  return new Promise((resolve, reject) => {
      const queryPlaca = `select * from cotizacionesporcentajes where cotizacionId = ? order by id desc;`;
      mysqlConnection.query(queryPlaca,[cotizacionId], (err, rows, fields) => {        
          if(!err){               
              resolve(rows)
          }else{
              console.log(err)
              reject(err)
          }
      })
  })
}


getCotizacionesByDdsesiongrabacion = (idsesiongrabacion) => {
  return new Promise((resolve, reject) => {
      const queryPlaca = `select * from cotizaciones where idsesiongrabacion = ? order by id desc;`;
      mysqlConnection.query(queryPlaca,[idsesiongrabacion], (err, rows, fields) => {        
          if(!err){               
              resolve(rows)
          }else{
              console.log(err)
              reject(err)
          }
      })
  })
}

getCotizacion = (cotizacionId) => {
    return new Promise((resolve, reject) => {
        const queryPlaca = `select * from cotizaciones where  id = ?;`;
        mysqlConnection.query(queryPlaca,[cotizacionId], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}
getCotizacionElementos = (cotizacionId) => {
  return new Promise((resolve, reject) => {
      const queryPlaca = `select * from cotizacioneselementos where  cotizacionId = ? order by id desc;`;
      mysqlConnection.query(queryPlaca,[cotizacionId], (err, rows, fields) => {        
          if(!err){               
              resolve(rows)
          }else{
              console.log(err)
              reject(err)
          }
      })
  })
}
 

updateCotizacion  = async (id, tag, tax, currency, destinatarioNombre, destinatarioCorreo,  destinatarioTelefono) => {
  return new Promise((resolve, reject) => {
    const query = 'UPDATE cotizaciones SET tag = ?, tax = ?, currency = ?, destinatarioNombre = ?, destinatarioCorreo = ?, destinatarioTelefono = ?  WHERE id = ?' ;
    mysqlConnection.query(query, [tag, tax, currency, destinatarioNombre, destinatarioCorreo,  destinatarioTelefono, id], function (err,rows, result) {   
        if(!err){           
            resolve({
              codigo: 0
            })
        }else{
          
            console.log(err)
            reject(err)
        }
    })
})
}


// (id, tag, tax, currency, destinatarioNombre, destinatarioCorreo,  destinatarioTelefono)
updateCotizacionAndElements = async (idsesiongrabacion, cotizacionId, body) => {
  let { inpt_userName, inpt_email, inpt_phone, inpt_tag, inpt_tax } = body;
  let tax = await convertStringToNumber(inpt_tax);
  let currency = body.chbx_currency == "pesos" ? 1 : 2;
  let { total, elementsToInsert, elementsToUpdate  }= await getTotalFromForm(body);

  await updateCotizacion(cotizacionId, inpt_tag, tax, currency, inpt_userName, inpt_email, inpt_phone)

  for(let element of elementsToUpdate)
  {  
    await updateElememtoCotizacion(element.cantidad, element.ps, element.pUnitario, element.total, element.id)
  }
  for(let element of elementsToInsert)
  {    
    await addElementoCotizacion(element.cantidad, element.ps, element.pUnitario, cotizacionId, element.total, 1, null);
  }
}

addCitizacion = async (idsesiongrabacion, body) =>
{
  //(cantidad, productoOServicio, precioUnitario, cotizacionId)
  let { inpt_userName, inpt_email, inpt_phone, inpt_tag } = body;
  let tax = await convertStringToNumber(body.inpt_tax);
  let currency = body.chbx_currency == "pesos" ? 1 : 2;
  let { total, elementsToInsert }= await getTotalFromForm(body);
  let response = await addCotizacionSimple(idsesiongrabacion, 6, inpt_tag, tax, currency, inpt_userName, inpt_email, inpt_phone, total, 1)
  for(let element of elementsToInsert)
  {
    await addElementoCotizacion(element.cantidad, element.ps, element.pUnitario, response.insertedID, element.total, 1, null);
  }
  return  response.insertedID;
}
 
 

const getTotalFromForm  = async(body) => {
    console.log(body)
    let total = 0;
    let counter = 0;
    let elementsToInsert = [];
    let elementsToUpdate = [];
    let idsToForOf = [];
    Object.keys(body).forEach((propiedad) => {
      if(propiedad.includes('inpt_ps') )
      {
        counter++;
      }          
      else if(propiedad.includes('inpt_bdId_ps'))
      {
        idsToForOf.push(obtenerNumeroOcero(propiedad));
      }
  });
  for(let x = 1; x <= counter; x++)
  {      
    let cantidad = convertStringToNumber(body[`inpt_c_${x}`]);
    let productoOServicio = body[`inpt_ps_${x}`];
    let pUnitario = convertStringToNumber(body[`inpt_pu_${x}`]);
    let valor = multiplicarConDecimal(cantidad, pUnitario);
    total = total + valor;
    elementsToInsert.push({ id: 0, mId: x, ps: productoOServicio, pUnitario, cantidad, total: valor  })
  }
  for(let id of idsToForOf)
  {  
    let cantidad = convertStringToNumber(body[`inpt_bdId_c_${id}`]);
    let productoOServicio = body[`inpt_bdId_ps_${id}`];
    let pUnitario = convertStringToNumber(body[`inpt_bdId_pu_${id}`]);
    let valor = multiplicarConDecimal(cantidad, pUnitario);
    total = total + valor;
    elementsToUpdate.push({ id: id, mId: 0, ps: productoOServicio, pUnitario, cantidad, total: valor  })
  }

  return { total, elementsToInsert, elementsToUpdate };

}

existAutomaticCotizacion = async (idsesiongrabacion) => {
  return new Promise((resolve, reject) => {
    const queryPlaca = `select * from cotizaciones where idsesiongrabacion = ? and removable = 0 ;`;
    mysqlConnection.query(queryPlaca,[idsesiongrabacion], (err, rows, fields) => {        
        if(!err){               
            resolve(rows)
        }else{
            console.log(err)
            reject(err)
        }
    })
})
}


existElementInCotizacion = async (service_id, cotizacionId) => {
  return new Promise((resolve, reject) => {
    const queryPlaca = `SELECT * FROM cotizacioneselementos where service_id = ? and cotizacionId = ? and modifiable = 0;`;
    mysqlConnection.query(queryPlaca,[service_id, cotizacionId], (err, rows, fields) => {        
        if(!err){               
            resolve(rows)
        }else{
            console.log(err)
            reject(err)
        }
    })
})
}



addCotizacionSimple = async (idsesiongrabacion, adminUserId, tag, tax, currency, destinatarioNombre, destinatarioCorreo, destinatarioTelefono, total, removable) => {
  return new Promise((resolve, reject) => {
        const query = `INSERT INTO cotizaciones (idsesiongrabacion, createdAt, AdminUserId, tag, tax, currency, destinatarioNombre, destinatarioCorreo, destinatarioTelefono, estatus_id, total, removable) 
        VALUES (?, now(), ?, ?, ?, ?, ?, ?, ?, ?, ?, ?);`;    
        mysqlConnection.query(query,[idsesiongrabacion, adminUserId, tag, tax, currency, destinatarioNombre, destinatarioCorreo, destinatarioTelefono, 1, total, removable], (error, results) => { 
            if(!error){               
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }else{
                console.log(error)
                reject(error)
            }
        })
    })
}
addElementoCotizacion = async (cantidad, productoOServicio, precioUnitario, cotizacionId, total, modifiable, service_id, isAditionalFromForm) => {
  let isAditional = isAditionalFromForm ?? false;
  return new Promise((resolve, reject) => {
        const query = `INSERT INTO cotizacioneselementos (cantidad, productoOServicio, precioUnitario, total,  createdAt, cotizacionId, modifiable, service_id, isAditional) 
        VALUES (?, ?, ?, ?, now(), ?, ?, ?, ?);`;    
        mysqlConnection.query(query,[cantidad, productoOServicio, precioUnitario, total, cotizacionId, modifiable, service_id, isAditional], (error, results) => { 
            if(!error){               
                const lastInsertedId = results.insertId;              
                resolve({
                    insertedID: lastInsertedId
                })
            }else{
                console.log(error)
                reject(error)
            }
        })
    })
}

UpdateElementoCotizacion = async (cantidad, precioUnitario, total, cotizacionElementoId) => {
  return new Promise((resolve, reject) => {
        const query = `update cotizacioneselementos set cantidad = ?, precioUnitario = ?, total = ? where id = ?;`;    
        mysqlConnection.query(query,[cantidad, precioUnitario, total, cotizacionElementoId], (error, results) => { 
            if(!error){               
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }else{
                console.log(error)
                reject(error)
            }
        })
    })
}

getElementsToDelete = async (id) => {
  let query = `select elementos.id from cotizacioneselementos elementos
  left join cotizaciones 
  on elementos.cotizacionId = cotizaciones.id
  where idsesiongrabacion = ? and removable = 0 and (isAditional = false or isAditional = null)
  and service_id not in(
  SELECT horarios.servicio_id FROM sesiongrabacionhorarios horarios
  left join sesiongrabacionfechas fechas 
  on horarios.sesiongrabacionfechas_id = fechas.id
  where fechas.sesiongrabacion_id = ?
  group by horarios.servicio_id
  )`;
  return new Promise((resolve, reject) => {    
    mysqlConnection.query(query, [id, id], (err, rows, fields) => {
      if(!err){               
        resolve(rows)
    }else{
        console.log(err)
        reject(err)
    }
      });
})
}
getElementsToDelete_aditionalServices = async (id) => {
  let query = `
  select elementos.id from cotizacioneselementos elementos
  left join cotizaciones coti
  on elementos.cotizacionId = coti.id
  where coti.idsesiongrabacion = ? and elementos.isAditional = 1
  and elementos.service_id not in(select reverseservicios.servicio_id from reverseservicios where reverseservicios.sesiongrabacion_id = ?);`;
  return new Promise((resolve, reject) => {    
    mysqlConnection.query(query, [id, id], (err, rows, fields) => {
      if(!err){               
        resolve(rows)
    }else{
        console.log(err)
        reject(err)
    }
      });
})
}

updateCotizacionFileName = async (id, fileName) => {
  return new Promise((resolve, reject) => {
    const query = 'UPDATE cotizaciones SET fileName = ? WHERE id = ?' ;
    mysqlConnection.query(query, [fileName, id], function (err,rows, result) {   
        if(!err){
           
            resolve({
              codigo: 0
            })
        }else{
          
            console.log(err)
            reject(err)
        }
    })
})
}



updateElememtoCotizacion = async (cantidad, productoOServicio, precioUnitario, total, id) => {

  return new Promise((resolve, reject) => {

      const query = 'UPDATE cotizacioneselementos SET cantidad = ?, productoOServicio = ?, precioUnitario = ?, total = ?   WHERE id = ?' ;
  
      mysqlConnection.query(query, [cantidad, productoOServicio, precioUnitario, total, id], function (err,rows, result) {   
          if(!err){
             
              resolve({
                codigo: 0
              })
          }else{
            
              console.log(err)
              reject(err)
          }
      })
  })
}
const obtenerNumeroOcero = (inputValue) => {
  var numerosExtraidos = inputValue.match(/\d+/);
  if (numerosExtraidos !== null) {
      return parseInt(numerosExtraidos[0]);
  } else {
      return 0;
  }
}
const  multiplicarConDecimal = (strNumero1, strNumero2) => {
  try {
      var numero1 = parseFloat(strNumero1);
      var numero2 = parseFloat(strNumero2);
      if (isNaN(numero1) || isNaN(numero2)) {
          return NaN;
      }
      var resultado = numero1 * numero2;
      resultado = Math.round(resultado * 10) / 10;
      return resultado;
  } catch (error) {
      return 0;
  }
}

const  convertStringToNumber = (str) => {
  try {
      let numero = parseFloat(str);      
      if (isNaN(numero)) {
          return 0;
      }
      let resultado = Math.round(numero * 10) / 10;
      return resultado;
  } catch (error) {
      return 0;
  }
}



const deleteCotizacion = (id) => {
  return new Promise((resolve, reject) => {
    const queryPlaca = `delete from cotizaciones where id = ?;`;
    mysqlConnection.query(queryPlaca, [id], (err, result) => {
        if (err) {              
          const response = {
              code: -1,
              mensaje: "Error al eliminar el registro"
              };
  
              resolve(response)

        } else {
          const response = {
              code: 0,
              mensaje: "Registro creado exitosamente"
          };

          resolve(response)
        }
      });
})
} 


const deleteCotizacionElements = (id) => {
  return new Promise((resolve, reject) => {
    const queryPlaca = `delete from  cotizacioneselementos where cotizacionId = ?; `;
    mysqlConnection.query(queryPlaca, [id], (err, result) => {
        if (err) {              
          const response = {
              code: -1,
              mensaje: "Error al eliminar el registro"
              };
  
              resolve(response)

        } else {
          const response = {
              code: 0,
              mensaje: "Registro eliminado exitosamente"
          };

          resolve(response)
        }
      });
})
} 

const deleteElementById = (id) => {
  return new Promise((resolve, reject) => {
    const queryPlaca = `delete from  cotizacioneselementos where id = ?; `;
    mysqlConnection.query(queryPlaca, [id], (err, result) => {
        if (err) {              
          const response = {
              code: -1,
              mensaje: "Error al eliminar el registro"
              };
  
              resolve(response)

        } else {
          const response = {
              code: 0,
              mensaje: "Registro eliminado exitosamente"
          };

          resolve(response)
        }
      });
})
} 

const addPercentageToCotizaciones = async(year, month, admin, percentage, justification) => {
  try {
    let sesions = await sesiongrabacionModelo.getSesionsToPayByYearMonthUser(year, month, admin);    
    for(let sesionForId of sesions)   
    {
        let sesionAsList = await sesiongrabacionModelo.getSesionToPayById(sesionForId.idsesiongrabacion);
        let sesion = sesionAsList[0];
        sesion["cotizaciones"] = [];        
        let cotizacionesList = await getCotizacionesByDdsesiongrabacion(sesionForId.idsesiongrabacion);
        for(let cotizacionToGetId of cotizacionesList)
        {
          await addPercentageToCotizacion(cotizacionToGetId.id, percentage, justification, 1)        
        }
    }
    return true;
  } catch (error) {
    return false;
  }
  
}



addPercentageToCotizacion = async (id, percentage, justification, isGeneral) => {
  return new Promise((resolve, reject) => {
        const query = `INSERT INTO cotizacionesporcentajes (percentage, justification, timestamp, cotizacionId, isGeneral) 
        VALUES (?, ?, now(), ?, ?);`;    
        mysqlConnection.query(query,[percentage, justification, id, isGeneral], (error, results) => { 
            if(!error){               
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }else{
                console.log(error)
                reject(error)
            }
        })
    })
}


module.exports = {
    addCitizacion,    
    addCotizacionSimple,
    getAllCotizacionData,
    updateCotizacionAndElements,
    getCotizacionesByDdsesiongrabacion,
    getCotizacionElementos,
    getCotizacionElementosForPdf,
    getTotalsForPdf,
    getCustomerDatForPdf,
    getTotals,
    getCotizacionValidity,
    getContactTable,
    updateCotizacionFileName,
    getCotizacion,
    addElementoCotizacion,
    UpdateElementoCotizacion,
    deleteCotizacion,
    deleteCotizacionElements,
    existAutomaticCotizacion,
    existElementInCotizacion,
    getElementsToDelete,
    getElementsToDelete_aditionalServices,
    deleteElementById,
    addPercentageToCotizaciones
}