const bcrypt = require('bcrypt');
const mysqlConnection = require('../config/database');


getActiveServices =  async () => {
    const query = 'SELECT * FROM servicios where isActive = 1 and byHour = 1 order by nombreServico desc;'
    return new Promise((resolve, reject) => {
        mysqlConnection.query(query, (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })    
}




addService= async (servicio_id,sesiongrabacion_id) => {
    return new Promise((resolve, reject) => {
        const query = 'INSERT INTO reverseservicios (servicio_id,sesiongrabacion_id) VALUES (?, ?)';
        mysqlConnection.query(query, [servicio_id, sesiongrabacion_id], (error, results) => {
            if (error) {
                    console.error('Error inserting availability:', error);
                    reject(error)
            } else {
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }
        });
    })
}



deleteServiceById = async (id) => {
    return new Promise((resolve, reject) => {
        const query = 'delete from servicios where id = ? ;';
        mysqlConnection.query(query, [id], (err, result) => {
            if (err) {
              console.log(err)
              const response = {
                  code: -1,
                  mensaje: "Error al eliminar el registro"
                  };
      
                  resolve(response)
  
            } else {
              const response = {
                  code: 0,
                  mensaje: "Registro creado exitosamente"
              };
  
              resolve(response)
            }
          });
})
}

deleteService= async (id, sesionId) => {
    return new Promise((resolve, reject) => {
            const query = 'delete from reverseservicios where servicio_id = ? and sesiongrabacion_id = ?;';
            mysqlConnection.query(query, [id, sesionId], (err, result) => {
                if (err) {
                  console.log(err)
                  const response = {
                      code: -1,
                      mensaje: "Error al eliminar el registro"
                      };
          
                      resolve(response)
      
                } else {
                  const response = {
                      code: 0,
                      mensaje: "Registro creado exitosamente"
                  };
      
                  resolve(response)
                }
              });
    })
}


getCheckedServices= (id) => {
    return new Promise((resolve, reject) => {
        const queryPlaca = "select * from reverseservicios where sesiongrabacion_id = ? ";
        mysqlConnection.query(queryPlaca,[id], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}

getCheckedServicesJoined = (id) => {
    return new Promise((resolve, reject) => {
        const queryPlaca = 
        `
        select 
reverseservicios.id as id, 
reverseservicios.servicio_id as servicio_id,  
reverseservicios.sesiongrabacion_id as sesiongrabacion_id ,
servicios.nombreServico
from reverseservicios 
left join servicios
on reverseservicios.servicio_id = servicios.id
where sesiongrabacion_id = ?
order by servicios.nombreServico asc;
        `;
        mysqlConnection.query(queryPlaca,[id], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}



deleteAllServicesBySesiondegrabacionId = async (sesiongrabacion_id) => {
    return new Promise((resolve, reject) => {
            const query = 'delete from reverseservicios where sesiongrabacion_id = ?;';
            mysqlConnection.query(query, [sesiongrabacion_id], (err, result) => {
                if (err) {
                  console.log(err)
                  const response = {
                      code: -1,
                      mensaje: "Error al eliminar el registro"
                      };
          
                      resolve(response)
      
                } else {
                  const response = {
                      code: 0,
                      mensaje: "Registro creado exitosamente"
                  };
      
                  resolve(response)
                }
              });
    })
}

getServiceById = async (id) => {
    return new Promise((resolve, reject) => {
        const queryPlaca = "select * from servicios where id = ? ";
        mysqlConnection.query(queryPlaca,[id], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}

getServiciosPrecios = async (servicio_id, hours) => {
    const query = `select * from serviciosPrecios where isActive = true and servicio_id = ? and (? >= hoursNeededMin and  ? <= hoursNeededMax)`
    return new Promise((resolve, reject) => {        
        mysqlConnection.query(query,[servicio_id,hours, hours], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}


getUnitaryCostByQuote = async (idsesiongrabacion) => {
    let query = `
    SELECT  elementos.service_id, elementos.precioUnitario  FROM cotizacioneselementos elementos
    left join cotizaciones
    on elementos.cotizacionId = cotizaciones.id
    where cotizaciones.idsesiongrabacion = ?
    and removable = 0;`;
    return new Promise((resolve, reject) => {        
        mysqlConnection.query(query,[idsesiongrabacion], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}   

getServiciosPreciosForCalculator = async (idsesiongrabacion, servicio_id, hours) => {

    const query = `select * from serviciosPrecios where isActive = true and servicio_id = ? and (? >= hoursNeededMin and  ? <= hoursNeededMax)`
    return new Promise((resolve, reject) => {        
        mysqlConnection.query(query,[servicio_id,hours, hours], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}


getPricesByServiceId = async (servicio_id) => {
    const query = `select * from serviciosPrecios where isActive = true and servicio_id = ? ;`
    return new Promise((resolve, reject) => {        
        mysqlConnection.query(query,[servicio_id], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}



addNewService = async (name, byHour, cost) => {
    let costToAdd = cost ?? 0;
    return new Promise((resolve, reject) => {
        const query = 'INSERT INTO servicios (nombreServico,isActive, costoHora, byHour) VALUES (?, 1, ?, ?)';
        mysqlConnection.query(query, [name, costToAdd, byHour], (error, results) => {
            if (error) {
                    console.error('Error inserting availability:', error);
                    reject(error)
            } else {
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }
        });
    })
}

addServicePriceDefault = async (createdBy_UserId, servicio_id) => {
    return new Promise((resolve, reject) => {
        const query = `INSERT INTO serviciosPrecios (precio, justificacion, hoursNeededMin, hoursNeededMax, createdBy_UserId, createdAt, isActive, servicio_id, currency) 
        VALUES (0, "NA", 0, 9999, ?, now(), 1, ?, 2)`;
        mysqlConnection.query(query, [createdBy_UserId, servicio_id], (error, results) => {
            if (error) {
                    console.error('Error inserting availability:', error);
                    reject(error)
            } else {                
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }
        });
    })
}

addServicePrice = async (precio, hoursNeededMin, hoursNeededMax, createdBy_UserId, servicio_id) => {
    return new Promise((resolve, reject) => {
        const query = `INSERT INTO serviciosPrecios (precio, justificacion, hoursNeededMin, hoursNeededMax, createdBy_UserId, createdAt, isActive, servicio_id, currency) 
        VALUES (?, "NA", ?, ?, ?, now(), 1, ?, 2)`;
        mysqlConnection.query(query, [precio, hoursNeededMin, hoursNeededMax, createdBy_UserId, servicio_id], (error, results) => {
            if (error) {
                    console.error('Error inserting availability:', error);
                    reject(error)
            } else {                
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }
        });
    })
}



addServicePrice = async (precio, hoursNeededMin, hoursNeededMax, createdBy_UserId, servicio_id) => {
    return new Promise((resolve, reject) => {
        const query = `INSERT INTO serviciosPrecios (precio, justificacion, hoursNeededMin, hoursNeededMax, createdBy_UserId, createdAt, isActive, servicio_id, currency) 
        VALUES (?, "NA", ?, ?, ?, now(), 1, ?, 2)`;
        mysqlConnection.query(query, [precio, hoursNeededMin, hoursNeededMax, createdBy_UserId, servicio_id], (error, results) => {
            if (error) {
                    console.error('Error inserting availability:', error);
                    reject(error)
            } else {                
                const lastInsertedId = results.insertId;
                resolve({
                    insertedID: lastInsertedId
                })
            }
        });
    })
}



updateServicePrice = async (precio, hoursNeededMin, hoursNeededMax, servicio_id) => {
    return new Promise((resolve, reject) => {
        const query = 'update serviciosPrecios set precio = ?, hoursNeededMin = ?, hoursNeededMax = ? where id = ?;';
        mysqlConnection.query(query, [precio, hoursNeededMin, hoursNeededMax, servicio_id], (error) => {
            if (error) {
                    console.error('Error inserting availability:', error);
                    reject(error)
            } else {                
                resolve({
                    codigo: 0
                })
            }
        });
    })
}
updateService= (id, nombreServicio, isActive, costoHoraFromForm) => {
    let costoHora = parseFloat(costoHoraFromForm ?? 0);
    return new Promise((resolve, reject) => {
       
        const sql = 'UPDATE servicios SET nombreServico = ?, isActive = ?, costoHora = ? WHERE id = ?;';
         mysqlConnection.query(sql, [nombreServicio, isActive, costoHora, id], (err, result) => {
        if (err) {
            console.log(err)
            const response = {
                code: -1,
                mensaje: "Error al crear el registro"
                };
    
                resolve(response)

        } else {
      
            const response = {
                code: 0,
                mensaje: "Registro creado exitosamente"
                };
    
                resolve(response)
        }
     });
    
 });
}


inactivatePricesNotIn = async (servicio_id, ids) => {
    return new Promise((resolve, reject) => {
        const query = `update serviciosPrecios set isActive = false where servicio_id = ? and id not in(${ids});`;
        mysqlConnection.query(query, [servicio_id], (error) => {
            if (error) {
                    console.error('Error inserting availability:', error);
                    reject(error)
            } else {
                
                resolve({
                    codigo: 0
                })
            }
        });
    })
}




getAditionalServicesBy_SesiondegrabacionId = async (sesiongrabacion_id) => {
    const query = ` select reverseservicios.id reverseservicios_id, servicios.id service_id, servicios.nombreServico, servicios.costoHora  from reverseservicios 
    left join servicios
    on reverseservicios.servicio_id = servicios.id
    where 
    byHour = 0 and
    reverseservicios.sesiongrabacion_id = ?;`
    return new Promise((resolve, reject) => {        
        mysqlConnection.query(query,[sesiongrabacion_id], (err, rows, fields) => {        
            if(!err){               
                resolve(rows)
            }else{
                console.log(err)
                reject(err)
            }
        })
    })
}


module.exports = {
    getActiveServices,
    addService,    
    deleteService,
    getCheckedServices,
    getCheckedServicesJoined,
    deleteAllServicesBySesiondegrabacionId,
    getServiceById,
    getServiciosPrecios,
    getServiciosPreciosForCalculator,
    addNewService,
    updateService,
    getPricesByServiceId,
    addServicePriceDefault,
    addServicePrice,
    updateServicePrice,
    getUnitaryCostByQuote,
    inactivatePricesNotIn,
    getAditionalServicesBy_SesiondegrabacionId,
    deleteServiceById
}