const moment = require('moment');
const  datesModelo  = require('../models/dates');
const  horariosModelo  = require('../models/horarios');
const  asisitentesmusicosModelo  = require('../models/asisitentesmusicos');
const  asistentesNoMusicos  = require('../models/asistentesNoMusicos');
const  serviciosModelo  = require('../models/servicios');
const  serviciosAdicionalesModelo  = require('../models/serviciosAdicionales');
const  equipoestudioModelo  = require('../models/equipoestudio');
const  sesiongrabacionModelo  = require('../models/sesiongrabacion');
const  cotizacionesModelo  = require('../models/cotizaciones');
//getHorariosByDateId

const deleteSesionDeGrabacion = async(idsesiongrabacion) => {
    let result1 = await deleteFechasYHorarios(idsesiongrabacion);
    if(result1.result)
    {
        await asisitentesmusicosModelo.deleteMusicosBySesiondegrabacionId(idsesiongrabacion)
        await asistentesNoMusicos.deleteNoMusicosBySesiondegrabacionId(idsesiongrabacion)
        await serviciosModelo.deleteAllServicesBySesiondegrabacionId(idsesiongrabacion)
        await serviciosAdicionalesModelo.deleteAllServicesBySesiondegrabacionId(idsesiongrabacion)
        await equipoestudioModelo.deleteAllEquipoEstudioBySesiongrabacion_id(idsesiongrabacion)
        await sesiongrabacionModelo.deleteSesionDeGrabacionById(idsesiongrabacion)
    }
    return { result: true }
}

const deleteFechasYHorarios = async (idsesiongrabacion) => {
    try {
        let dates = await datesModelo.getDatesBySesionGrabacionId(idsesiongrabacion)
        for(let dt of dates)
        {
            let id = dt.id;
            await horariosModelo.deleteHorario(id)
            await datesModelo.deleteFechas(id)
        }
        return { result: true }
    } catch (error) {
        console.log(error);
        return { result: false }
    }
 
}


const getFechaYHorarioResumidos = async (sesiongrabacion_id) => {    
    let toReturn = [];
    let dates = await datesModelo.getDatesBySesionGrabacionId(sesiongrabacion_id)
    for(let dt of dates)
    {
        let horarios = await horariosModelo.getHorariosByDateId(dt.id)        
        let segmentos = getSegmentos(horarios)        
        let horariosResumidos = getHorariosByDate(segmentos)
        let servicios = getServicios(horarios);
        let fecha = moment(dt.date).locale('es-us').format("ll")
        toReturn.push({ id: dt.id, date: fecha, sesiongrabacion_id, horarios: horariosResumidos, location: dt.name, servicios })
    }
    return toReturn;
}

const getServicios = (list) => {
    let serviciosList = [];
    for(let item of list)
    {
        if(!serviciosList.includes(item.nombreServico))
        {
            serviciosList.push(item.nombreServico);
        }
    }
    return serviciosList ? serviciosList.join(', ') : "";
}


const getHorariosByDate = (segmentos) => {
    let horarios = [];
    for(let segmento of segmentos)
    {
        let sLength = segmento.length
        horarios.push(`${segmento[0].inicio} -> ${segmento[(sLength - 1)].fin}`)
    }
    return horarios.join(', ');
}

const getSegmentos = (horarios) => {
    let listaDeSegmentos = [];
    let segmento = [];
    for(let horario of horarios)
    {
        let segmentoLength = segmento.length;
        if(segmentoLength == 0)
        {
            segmento.push(horario)
        }
        else if(segmento[segmentoLength - 1].order === (horario.order - 1))
        {
            segmento.push(horario)
        }    
        else
        {
            listaDeSegmentos.push(segmento);
            segmento = [];
            segmento.push(horario)
        }           
    }
    return listaDeSegmentos.length == 0 && segmento.length !==0 ? [ segmento ] : listaDeSegmentos;
}

//(horario_id, sesiongrabacionfechas_id, servicio_id)
const insertDateTimeServices = async(req) =>
{
    let { inpt_Selected_idsesiongrabacion, inpt_Selected_Location, inpt_Date} = req.body;
    console.log(req.body)
    let dd_Service = parseInt(req.body.inpt_Selected_Service);
    let dtResult = await datesModelo.addDate(inpt_Selected_idsesiongrabacion,inpt_Date, inpt_Selected_Location)    
    for (const [key, value] of Object.entries(req.body)) {   
        if(key.includes("checkbox_"))
        {
            let horario_id = parseInt(value);
            let servicio_id = 0;
            if(dd_Service === 0)
            {
                servicio_id = parseInt(req.body[`dd_Service_${horario_id}`])
            }
            else
            {
                servicio_id = dd_Service;
            }
            await horariosModelo.addHorario(horario_id, dtResult.insertedID, servicio_id)        
        }        
    }
    return true;
}


// {
//     id: 246,
//     horario_id: 6,
//     sesiongrabacionfechas_id: 173,
//     createdAt: 2024-01-24T01:36:35.000Z,
//     updatedAt: 2024-01-24T01:36:35.000Z,
//     sesiongrabacionhorarioscol: null,
//     servicio_id: 1,
//     label: '12 pm - 1 pm',
//     order: 6,
//     inicio: '12 pm',
//     fin: '1 pm'
//   },

const updateDateTimeServices = async(req) =>
{
    let {  inpt_fechaId } = req.body;
    let fechaId = parseInt(inpt_fechaId)
    await horariosModelo.deleteHorario(fechaId)
    for (const [key, value] of Object.entries(req.body)) { 
        if(key.includes("checkbox_"))
        {
            let horario_id = parseInt(value);
            let servicio_id = 0;    
            servicio_id = parseInt(req.body[`dd_Service_${horario_id}`])         
            await horariosModelo.addHorario(horario_id, fechaId, servicio_id)        
        }    
    }   
    return true;
}

const getNameEmailAndPhoneForCotizacion = async (sesiongrabacion_id, destinatarioNombre, destinatarioCorreo, destinatarioTelefono) => {
    if(destinatarioNombre !== null && destinatarioNombre !== undefined 
        && destinatarioCorreo !== null && destinatarioCorreo !== undefined
        && destinatarioTelefono !== null && destinatarioTelefono !== undefined )
        {
            return { destinatarioNombreToUse: destinatarioNombre, destinatarioCorreoToUse: destinatarioCorreo, destinatarioTelefonoToUse: destinatarioTelefono  }
        }
        else
        {
            let sesions = await sesiongrabacionModelo.getSesiongrabacion(sesiongrabacion_id);
            let sesion = sesions[0];
            return {
                destinatarioNombreToUse: sesion.nombreBandaGrupo ?? null,
                destinatarioCorreoToUse: sesion.destinatarioCorreo ?? null,
                destinatarioTelefonoToUse: sesion.destinatarioTelefono ?? null,
            }
        }
}
const getPrice = async(servicio_id, qty) => {
    let totalHrs = parseFloat(qty);
    let precios = await serviciosModelo.getServiciosPrecios(servicio_id, totalHrs)    
    if(precios && precios.length == 1)
    {
        let total = parseFloat(precios[0].precio) * totalHrs;
        return { result: true,  total, totalHrs, precio: precios[0].precio };
    }
    else
    {
        return {  result: false, total, totalHrs, precio: 0 };
    }
}

// const generateAutomaticQuoteOG = async(sesiongrabacion_id, adminUserId, destinatarioNombre, destinatarioCorreo, destinatarioTelefono) => {    
//     let tag = "Cotización automática";
//     let tax = 0;
//     let currency = 2;
//     let total = 0;
//     let cotizacionId = 0;
//     let existRecords = await cotizacionesModelo.existAutomaticCotizacion(sesiongrabacion_id);
//     let { destinatarioNombreToUse, destinatarioCorreoToUse, destinatarioTelefonoToUse } = await getNameEmailAndPhoneForCotizacion(sesiongrabacion_id, destinatarioNombre, destinatarioCorreo, destinatarioTelefono)
//     if(existRecords && existRecords.length > 0)
//     {
//         let cotizacion_id = existRecords[0].id;
//         let services = await sesiongrabacionModelo.getHoursByServiceBySesiongrabacion_id(sesiongrabacion_id);
//         for(let service of services)
//         {
//             let precios = await serviciosModelo.getServiciosPrecios(service.servicio_id, service.servicio_id)
//             let existElement = await cotizacionesModelo.existElementInCotizacion(service.servicio_id, cotizacion_id);
//             let { result, total, totalHrs }  = await getPrice(service.servicio_id, service.qty); 
//             if(result)
//             {
//                 if(existElement && existElement.length > 0)
//                 {
//                     await cotizacionesModelo.UpdateElementoCotizacion(totalHrs, precios[0].precio, total, existElement[0].id)                
//                 }
//                 else
//                 {                      
//                     await cotizacionesModelo.addElementoCotizacion(totalHrs, service.nombreServico, precios[0].precio, cotizacion_id, total, 0, service.servicio_id)                
//                 }
//             }           
//         }
//     }
//     else 
//     {        
//         let addCitizacionResult =  await cotizacionesModelo.addCotizacionSimple(sesiongrabacion_id, adminUserId, tag, tax, currency, destinatarioNombreToUse, destinatarioCorreoToUse, destinatarioTelefonoToUse, total, 0);
//         cotizacionId = addCitizacionResult.insertedID;
//         let horarios = await horariosModelo.getHoursByServiceBySesiongrabacion_id(sesiongrabacion_id)    
//         for(let hr of horarios)
//         { 
//             let { result, total, totalHrs, precio }  = await getPrice(hr.servicio_id, hr.qty);            
//             if(result)
//             {
//                 await cotizacionesModelo.addElementoCotizacion(totalHrs, hr.nombreServico, precio, cotizacionId, total, 0, hr.servicio_id)
//             }            
//         }
//     }  
//     await deleteNonExistentElements(sesiongrabacion_id);
//     return { newCotizacionId: cotizacionId }
// }

const generateAutomaticQuote = async(sesiongrabacion_id, adminUserId, destinatarioNombre, destinatarioCorreo, destinatarioTelefono) => {    
    let tag = "Cotización automática";
    let tax = 0;
    let currency = 2;
    let total = 0;
    let cotizacionId = 0;
    let existRecords = await cotizacionesModelo.existAutomaticCotizacion(sesiongrabacion_id);
    let { destinatarioNombreToUse, destinatarioCorreoToUse, destinatarioTelefonoToUse } = await getNameEmailAndPhoneForCotizacion(sesiongrabacion_id, destinatarioNombre, destinatarioCorreo, destinatarioTelefono)
    if(existRecords && existRecords.length > 0)
    {
         cotizacionId = existRecords[0].id;    
    }
    else 
    {        
        let addCitizacionResult =  await cotizacionesModelo.addCotizacionSimple(sesiongrabacion_id, adminUserId, tag, tax, currency, destinatarioNombreToUse, destinatarioCorreoToUse, destinatarioTelefonoToUse, total, 0);
        cotizacionId = addCitizacionResult.insertedID;
    }  
    await generateAutomaticQuote_updateElements(sesiongrabacion_id, cotizacionId);
    await generateAutomaticQuote_updateAditionalServices(sesiongrabacion_id, cotizacionId)
    await deleteNonExistentElements(sesiongrabacion_id);
    return { newCotizacionId: cotizacionId }
}



const generateAutomaticQuote_updateElements = async (sesiongrabacion_id, cotizacion_id) => {
    try {
        let services = await sesiongrabacionModelo.getHoursByServiceBySesiongrabacion_id(sesiongrabacion_id);
        for(let service of services)
        {
            let existElement = await cotizacionesModelo.existElementInCotizacion(service.servicio_id, cotizacion_id);
            let { result, total, totalHrs, precio }  = await getPrice(service.servicio_id, service.qty); 
                if(result)
                {
                    if(existElement && existElement.length > 0)
                    {
                        await cotizacionesModelo.UpdateElementoCotizacion(totalHrs, precio, total, existElement[0].id)                
                    }
                    else
                    {                      
                        await cotizacionesModelo.addElementoCotizacion(totalHrs, service.nombreServico, precio, cotizacion_id, total, 0, service.servicio_id)                
                    }
                }           
        }
        return true;
    } catch (error) {
        console.log(error)
        return false;
    }       
}

const generateAutomaticQuote_updateAditionalServices = async(sesiongrabacion_id, cotizacion_id) => { 
    try {
        let services = await serviciosModelo.getAditionalServicesBy_SesiondegrabacionId(sesiongrabacion_id)        
        for(let service of services)
        {     
            let existElement = await cotizacionesModelo.existElementInCotizacion(service.service_id, cotizacion_id);
            let unidades = 1;
            let price = service.costoHora;
            if(existElement && existElement.length > 0)
            {
                await cotizacionesModelo.UpdateElementoCotizacion(unidades, price, price, existElement[0].id)     
            }
            else
            {
                await cotizacionesModelo.addElementoCotizacion(unidades, service.nombreServico, price, cotizacion_id, price, 0, service.service_id, true)                
            }
        }
        return true;
    } catch (error) {
        console.log(error)
        return true;
    }
    

}
 


const deleteNonExistentElements = async (sesiongrabacion_id) => {
    let elements = await cotizacionesModelo.getElementsToDelete(sesiongrabacion_id);
    if(elements && elements.length > 0)
    {
        for(let element of elements)
        {
            await cotizacionesModelo.deleteElementById(element.id);
        }        
    }
    let aditionalElements = await cotizacionesModelo.getElementsToDelete_aditionalServices(sesiongrabacion_id);
    if(aditionalElements && aditionalElements.length > 0)
    {
        for(let element of aditionalElements)
        {
            await cotizacionesModelo.deleteElementById(element.id);
        }
        
    }
    return true;
}

const getDataToPayByUserAndService = async(year, month, user_id) => {
    let data = await sesiongrabacionModelo.getServicesToPayByUserId(parseInt(year), parseInt(month), parseInt(user_id));
    for(let dt of data)
    {
        let total = 0;            
        let precios = await serviciosModelo.getUnitaryCostByQuote(dt.idsesiongrabacion);                   
        if(precios && precios.length > 0)
        {
            let precio = precios.find(obj => obj.service_id === dt.servicio_id);            
            if(precio)
            {
                let totalHrs = parseFloat(dt.hours);
                total = parseFloat(precio.precioUnitario) * totalHrs;    
            }                 
        }
        dt["total"] = total;
    }
    return data; 

}



// (cantidad, productoOServicio, precioUnitario, cotizacionId, total)

// {
//     id: 1,
//     precio: 45,
//     justificacion: 'NA',
//     hoursNeededMin: 0,
//     hoursNeededMax: 9,
//     createdBy_UserId: 120,
//     createdAt: 2024-01-19T18:57:14.000Z,
//     isActive: 1,
//     servicio_id: 1,
//     currency: 2
//   }
//}
///updateServicePrice = async (precio, hoursNeededMin, hoursNeededMax, servicio_id)
const updatePricesByServiceId = async(service_id, user_id, body) => {
    let activeIds = [];
    const contador = Object.keys(body).filter(key => key.startsWith('inpt_min_')).length;    
    for(let i = 1; i < (contador + 1); i++)
    {
        let id = parseInt(body[`inpt_og_id_${i}`]);
        let min = parseInt(body[`inpt_min_${i}`])
        let max = parseInt(body[`inpt_max_${i}`])
        let price = parseInt(body[`inpt_cost_${i}`])        
        if(min != NaN && max != NaN && price != NaN)
        {
            if(id == 0)
            {
                let { insertedID } = await serviciosModelo.addServicePrice(price, min, max, user_id, service_id);
                if(insertedID)
                {
                    activeIds.push(insertedID)
                }
            }
            else
            {
                await serviciosModelo.updateServicePrice(price, min, max, id);
                activeIds.push(id)
            }
        }
        
    }
    if(activeIds && activeIds.length > 0)
    {
        const idsToUpdate = activeIds.join(', ');
        await serviciosModelo.inactivatePricesNotIn(service_id, idsToUpdate)
    }
    return { result: true }
}



const getSesionsToPay = async(year, month, user_id) => {
    let data = [];
    let sesions = await sesiongrabacionModelo.getSesionsToPayByYearMonthUser(year, month, user_id);    
    for(let sesionForId of sesions)   
    {
        let sesionAsList = await sesiongrabacionModelo.getSesionToPayById(sesionForId.idsesiongrabacion);
        let sesion = sesionAsList[0];
        sesion["cotizaciones"] = [];        
        let cotizacionesList = await cotizacionesModelo.getCotizacionesByDdsesiongrabacion(sesionForId.idsesiongrabacion);
        for(let cotizacionToGetId of cotizacionesList)
        {
            let cotizacion = await cotizacionesModelo.getAllCotizacionData(cotizacionToGetId.id);            
            sesion.cotizaciones.push(cotizacion);            
        }
        data.push(sesion);
    }
    return data;
}

module.exports = { deleteSesionDeGrabacion, getFechaYHorarioResumidos, insertDateTimeServices, updateDateTimeServices, generateAutomaticQuote, getDataToPayByUserAndService, updatePricesByServiceId, getSesionsToPay }