const fs = require("fs");
const PDFDocument = require("pdfkit-table");
const { PDFSPATH } = require('../config');

const generatePdf = async(list, data, customer, totals, msj, currencyMsj) => {

    console.log({ list, data, customer, totals, msj, currencyMsj })
    let doc = new PDFDocument({ margin: 30, size: 'A4' });
    const pdfsPath = PDFSPATH;
    const fileName = `bunkerstudios_${Date.now()}${Math.floor(Math.random() * 100000)}`;
    const stream = fs.createWriteStream(`${pdfsPath}${fileName}.pdf`);
    doc.pipe(stream);
    doc.image('./public/img/logo1.png', 0, 0, { width: 595.28});    
    
    const table = {
        title: "Descripción del servicio y cotización",
        subtitle: currencyMsj, 
        headers: [ "Producto o servicio", "Cantidad", "Precio unitario", "Total"],
        rows: list,
    };
      
    const tableTotals = {
        headers: [
            {label:"", property:"name",  },
            {label:"", property:"name", },           
        ], 
        rows: totals,
    };
    const tableDatos = {
        title: "Datos Generales",                                                                          
        headers: [
            {label:"Fecha de la emisión", property:"name", valign: "left", headerColor:"#fc5c28", headerOpacity:0.8, color: '#ffffff' },
            {label:"Teléfono", property:"name", valign: "left",   headerColor:"#fc5c28", headerOpacity:0.8 },
            {label:"Correo electrónico", property:"name", valign: "left",  headerColor:"#fc5c28", headerOpacity:0.8 },
            {label:"Domicilio", property:"name", valign: "left",  headerColor:"#fc5c28", headerOpacity:0.8 },
        ], 
        rows: data,
    };
 const tableCustomer = {
        title: "Datos del cliente",
        headers: [
            {label:"Nombre", property:"name", valign: "left", headerColor:"#fc5c28", headerOpacity:0.8, color: '#ffffff' },
            {label:"Correo electrónico", property:"name", valign: "left",  headerColor:"#fc5c28", headerOpacity:0.8 },
            {label:"Teléfono", property:"name", valign: "left",   headerColor:"#fc5c28", headerOpacity:0.8 },
        ], 
        rows: customer,
    };
    

    doc.moveDown(1); 
    await doc.table(tableDatos, { 
        width: 530,       
    });
   
    doc.moveDown(1); 
    await doc.table(tableCustomer, { 
        width: 530,
    });  

    doc.moveDown(3); 
    doc.fontSize(20);
    doc.text(`Cotización`, {
    width: 530,
    align: 'center'
    }
    );
   
    await doc.table(table, { 
        width: 530,
    });     
    await doc.table(tableTotals, { 
        width: 130,
        x: 430,
        hideHeader: true, 
    });     
    

    doc.moveDown(3); 
    doc.fontSize(8);
    doc.text(msj,
        30,
        790  
    );



    doc.end();
    const promise = new Promise((resolve) => {
    stream.on("finish", () => {
        resolve({ pdfsPath, fileName });
        })
    });
    return promise;   
}

const eliminarArchivo = (filePath) => {
    fs.unlink(filePath, (unlinkErr) => {
        if (unlinkErr) {
            console.error('Error al eliminar el archivo:', unlinkErr);
        } else {
            console.log('Archivo eliminado con éxito.');
        }
    });
};

module.exports = { generatePdf, eliminarArchivo }